package com.ningdatech.pmapi.projectlib.handle;

import java.util.Collection;
import java.util.List;
import java.util.Map;
import java.util.Objects;
import java.util.stream.Collectors;

import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.ningdatech.basic.model.PageVo;
import com.ningdatech.pmapi.common.statemachine.event.ProjectStatusChangeEvent;
import com.ningdatech.pmapi.projectlib.enumeration.ProjectStatusEnum;
import com.ningdatech.pmapi.projectlib.manage.AnnualPlanLibManage;
import com.ningdatech.pmapi.projectlib.model.entity.NdProjectStatusChange;
import com.ningdatech.pmapi.projectlib.model.entity.Project;
import com.ningdatech.pmapi.projectlib.model.req.ProjectListReq;
import com.ningdatech.pmapi.projectlib.model.vo.AnnualPlanListItemVO;
import com.ningdatech.pmapi.projectlib.service.INdProjectStatusChangeService;
import com.ningdatech.pmapi.projectlib.service.IProjectService;
import org.springframework.core.annotation.Order;
import org.springframework.stereotype.Component;

import com.ningdatech.pmapi.common.constant.CommonConstant;
import com.ningdatech.pmapi.projectlib.enumeration.StepStatusEnum;
import com.ningdatech.pmapi.projectlib.model.vo.ProcessDetailVO;

/**
 * 年度计划处理
 *
 * @author CMM
 * @since 2023/02/24 14:35
 */
@Order(5)
@Component
public class AnnualPlanHandle extends AbstractProcessBusinessHandle {
    private final AnnualPlanLibManage annualPlanLibManage;
    private final INdProjectStatusChangeService projectStatusChangeService;

    public AnnualPlanHandle(AnnualPlanLibManage annualPlanLibManage, INdProjectStatusChangeService projectStatusChangeService){
        this.annualPlanLibManage = annualPlanLibManage;
        this.projectStatusChangeService = projectStatusChangeService;
    }

    @Override
    void businessHandle(Long projectId, List<ProcessDetailVO> processSchedule) {
        ProcessDetailVO processDetailVO = new ProcessDetailVO();
        // 根据年度计划库列表是否包含该项目以及是否开启方案申报判断年度计划是否开始
        ProjectListReq req = new ProjectListReq();
        PageVo<AnnualPlanListItemVO> planLibList = annualPlanLibManage.annulPlanLibList(req);
        Collection<AnnualPlanListItemVO> records = planLibList.getRecords();
        Map<Long, AnnualPlanListItemVO> itemVOMap = records.stream()
                .collect(Collectors.toMap(AnnualPlanListItemVO::getProjectId, v -> v));
        AnnualPlanListItemVO itemVO = itemVOMap.get(projectId);
        // 年度计划库中不包含当前项目
        if (Objects.isNull(itemVO)){
            processDetailVO.setStepStatus(StepStatusEnum.NOT_START);
            processDetailVO.setProcessName(CommonConstant.ANNUAL_PLAN);
            processSchedule.add(processDetailVO);
            return;
        }
        Boolean isStartDeclaredProject = itemVO.getIsStartDeclaredProject();
        // 年度计划库中包含当前项目但未开启方案申报
       if (Boolean.FALSE.equals(isStartDeclaredProject)) {
            processDetailVO.setStepStatus(StepStatusEnum.NOT_START);
            processDetailVO.setProcessName(CommonConstant.ANNUAL_PLAN);
            processSchedule.add(processDetailVO);
            return;
        }
        // 年度计划库包含当前项目且开启了方案申报
        Integer status = itemVO.getStatus();
        if (ProjectStatusEnum.IN_THE_ANNUAL_PLAN.getCode().equals(status)){
            // 根据部门联审通过的时间获取
            NdProjectStatusChange projectStatusChange = projectStatusChangeService.getOne(Wrappers.lambdaQuery(NdProjectStatusChange.class)
                    .eq(NdProjectStatusChange::getProjectId, projectId)
                    .eq(NdProjectStatusChange::getEvent, ProjectStatusChangeEvent.DEPARTMENT_UNITED_REVIEW_PASS.name())
                    .last("limit 1"));
            processDetailVO.setFinishTime(projectStatusChange.getCreateOn());
            processDetailVO.setStepStatus(StepStatusEnum.ON_GOING);

        } else if (ProjectStatusEnum.BE_SUSPENDED.getCode().equals(status)) {
            // 根据年度计划暂缓的时间获取
            NdProjectStatusChange projectStatusChange = projectStatusChangeService.getOne(Wrappers.lambdaQuery(NdProjectStatusChange.class)
                    .eq(NdProjectStatusChange::getProjectId, projectId)
                    .eq(NdProjectStatusChange::getEvent, ProjectStatusChangeEvent.ANNUAL_PLAN_SUSPEND.name())
                    .last("limit 1"));
            processDetailVO.setFinishTime(projectStatusChange.getCreateOn());
            processDetailVO.setStepStatus(StepStatusEnum.REJECTED);
        } else {
            // 根据开启方案申报的时间获取
            NdProjectStatusChange projectStatusChange = projectStatusChangeService.getOne(Wrappers.lambdaQuery(NdProjectStatusChange.class)
                    .eq(NdProjectStatusChange::getProjectId, projectId)
                    .eq(NdProjectStatusChange::getEvent, ProjectStatusChangeEvent.ANNUAL_PLAN_PROJECT_OPEN_PLAN_DECLARE.name())
                    .last("limit 1"));
            processDetailVO.setFinishTime(projectStatusChange.getCreateOn());
            processDetailVO.setStepStatus(StepStatusEnum.COMPLETED);
        }
        processDetailVO.setProcessName(CommonConstant.ANNUAL_PLAN);
        processSchedule.add(processDetailVO);
    }
}
